/*
MATERIAL TOOLBOX
Created by Tim Little
Contact me with bug reports or feedback

timhlittle@gmail.com
timhlittle.prosite.com

Manage the Material assignment, Material ID number, and Wirecolor of scene objects.

Feature List:
+ Easily Select all objects with shared Material, Mat ID, or Wirecolor
+ Interactive Clipboard to store Materials, Mat ID's, or Wirecolors
+ Create a gradient of unique Wirecolors across selected objects, based on their Material assignment or Mat ID

Installation:
Drag and drop the downloaded MZP file on to an open window of 3ds Max. An installer dialog will open automatically.

VERSION 1.0

TO DO LIST:
+ Ability to rainbow a selection of objects with different material IDs

*/

--//=================================================================================================//--	
--//======================================== DEFINE VARIABLES =========================================//--

	global iconPath = (GetDir #scripts + "\\LittleScripts\\icons\\" + "MaterialToolbox_Banner.bmp")
	if not doesFileExist (iconPath) then iconPath = ""


--//=================================================================================================//--	
--//======================================= DEFINE FUNCTIONS =========================================//--

function getMatID obj=
(
	MatID_Face1 = undefined
				
	if ((classOf obj) == Editable_Mesh) then
	(
		ObjMesh = snapshotAsMesh obj
		MatID_Face1 = getFaceMatID ObjMesh 1
	)
	
	if ((classOf obj) == Editable_Poly) then
	(
		MatID_Face1 = polyop.getFaceMatID obj 1
	)
	
	MatID_Face1
)

function applyWireColorRainbow rainbowMethod selectedOrScene startColor=
(
	disableSceneRedraw()
	if selectedOrScene == 1 then
	(	
		selectedMatArray = #()
		
		-- Collect the Material & Material ID of the original selection
		for obj in selection do
		(
			--Check to see if user wants to sort by Material or Material ID
			if rainbowMethod == 1 then --Material
			(
				if obj.material != undefined then append selectedMatArray obj.material
			)
			
			if rainbowMethod == 2 then --Material ID
			(
				try
				(
					MatID_Face1 = getMatID obj -- call custom function
					
					append selectedMatArray MatID_Face1
				)
				catch(print"ERROR: COULD NOT OBTAIN MAT ID")
			)
			
			if rainbowMethod == 3 then -- per object
			(
				selectedMatArray = selection as array
			)
		)
	)
	
	try
	(
		
		hueVar = ceil(255/selectedMatArray.count)
		
		i = 1
		for mat in selectedMatArray do
		(
			selectedObjArray = #()
			
			if selectedOrScene == 1 then selectedObjArray = selection
			if selectedOrScene == 2 then selectedObjArray = objects
			
			for obj in selectedObjArray do
			(
				var = undefined
				
				if rainbowMethod == 1 then --Material
				(
					var = obj.material
				)
				if rainbowMethod == 2 then --Material ID
				(
					MatID_Face1 = getMatID obj -- call custom function
					
					var = MatID_Face1
				)
				
				if rainbowMethod == 3 then --Object
				(
					obj = mat -- just set it to equal, we want to affect every object
					
					objWireColorVar = startColor -- set Initial color to red
					
					objWireColorVar.hue = hueVar*i --shift hue
					
					obj.wireColor = objWireColorVar --assign new color to obj
				)
				else if var == mat then
				(
					objWireColorVar = startColor -- set Initial color to red
					
					objWireColorVar.hue = hueVar*i --shift hue
					
					obj.wireColor = objWireColorVar --assign new color to obj
				)
				
			)
			
			i = i+1 --iterate the index
		)
		enableSceneRedraw()
		gc()
	)
	catch
	(
		print"ERROR: FAILED TO APPLY RAINBOW"
		enableSceneRedraw()
		gc()
	)
)

gc()

--//==================================================================================================//--	
--//======================================== DEFINE ROLLOUTS ==========================================//--

	screenHeight = sysInfo.desktopSize[2] --the second value in the point2 is the height
	screenMiddle = screenHeight / 2

	if materialToolbox_MasterFloater != undefined then CloseRolloutFloater materialToolbox_MasterFloater
	materialToolbox_MasterFloater = newRolloutFloater "Material Toolbox" 225 180 380 (screenMiddle-450)

	
	rollout about_rollout "Version 1.0" category:1
	(
		bitmap Banner_Btmp "Bitmap" pos:[0,0] width:225 height:41 fileName:(iconPath)
		hyperlink bio_label "Tim Little" align:#left across:2 address:"timhlittle.prosite.com"  color:(color 208 118 57) hovercolor:(color 255 255 255) visitedcolor:(color 208 118 57)
		label email_label "timhlittle@gmail.com" align:#right
	)

	rollout matID_rollout "Material ID" category:2
	(
		-- LAYOUT UI
		
		button querySelectObject_btn "Display Material IDs of selection" height:30 width:180 offset:[0,0]
		spinner matIDToSelect_spinner "Mat ID: " range:[1,999,1] type:#integer scale:1 fieldwidth:50 align:#center offset:[0,0]	height:30
		button getMatIDofSelection_btn "Get Mat ID of selection" width:180
		
		button applyMatIDToObjects_btn "Apply Mat ID" height:35 width:180 offset:[0,10]
		checkbox instanceMaterialModifier_cbox "Instance Material Modifier?" checked:false align:#center
		checkbox excludeHiddenObjects_cbox "Exclude Hidden Objects?" checked:true align:#center
		
		button selectObjects_btn "Select Objects with Mat ID" height:30 width:180 offset:[0,10]
		checkbox appendToSelection_cbx "Append to Current Selection?" checked:false align:#center
		
		on selectObjects_btn pressed do
		(
			if excludeHiddenObjects_cbox.checked then objArray = (for obj in geometry where not obj.isHidden collect obj)
			else objArray = geometry
			
			objectsToselect = #()
			
			for obj in objArray do
			(
				MatID_Face1 = getMatID obj -- call custom function
				
				if MatID_Face1 == matIDToSelect_spinner.value then append objectsToselect obj
			)	
			
			undo on 
			(
				if appendToSelection_cbx.checked then selectmore objectsToselect else select objectsToselect
			)
			
		) --end of select_btn
		
		-- EVENT HANDLING
		on getMatIDofSelection_btn pressed do
		(
			obj = selection[1]
			MatID_Face1 = undefined
					
			try
			(
				MatID_Face1 = getMatID obj -- call custom function
				
				matIDToSelect_spinner.value = MatID_Face1 
			)
			catch(print"Could not get and set Mat ID of selection")
		)
		
		
		on applyMatIDToObjects_btn pressed do
		(
			if excludeHiddenObjects_cbox.checked then objArray = (for obj in selection where not obj.isHidden collect obj)
			else objArray = selection
			
			for i=1 to objArray.count where (superClassOf objArray[i] == GeometryClass or superClassOf objArray[i] == Shape and objArray[i].IsHidden == false) do
			(
				--print (classOf objArray[i])
				undo on
				(
					try
					(
						if objArray[i].modifiers[#Material].materialID == undefined then
						(
							if instanceMaterialModifier_cbox.checked then
							(
								if i == 1 then addmodifier objArray[i] (MaterialModifier())
								else
								(
									temp_mod = (objArray[i-1].modifiers[#Material])
									addmodifier objArray[i] temp_mod
								)
							)
							else addModifier objArray[i] (MaterialModifier())
								
							objArray[i].modifiers[#Material].materialID = matIDToSelect_spinner.value
						)
						else objArray[i].modifiers[#Material].materialID = matIDToSelect_spinner.value
					)
					catch
					(
						if instanceMaterialModifier_cbox.checked then
						(
							if i == 1 then addmodifier objArray[i] (MaterialModifier())
							else
							(
								temp_mod = (objArray[i-1].modifiers[#Material])
								addmodifier objArray[i] temp_mod
							)
						)
						else addModifier objArray[i] (MaterialModifier())
								
						objArray[i].modifiers[#Material].materialID = matIDToSelect_spinner.value
					)
				)
			)
		) --end of apply button
			
		on querySelectObject_btn pressed do
		(
			objMatIDString = ""
		
			for obj in selection where (superClassOf obj == GeometryClass or superClassOf obj == Shape) do
			(
				MatID_Face1 = getMatID obj -- call custom function

				objMatIDString = objMatIDString + (obj.name as string)
				objMatIDString = objMatIDString + " = "
				objMatIDString = objMatIDString + (MatID_Face1 as string)
				objMatIDString = objMatIDString + "\n"
			)

			print ("=========================================\n" + objMatIDString)
			
			if objMatIDString != "" then messageBox objMatIDString title: "Mat IDs of Selection" beep:false 
			else messageBox "Nothing is Selected" title: "Material ID's of Selection" beep:false

		) --end of query btn
		
		
		on matID_rollout rolledUp state do
		(
			if state == true then
				materialToolbox_MasterFloater.size =[materialToolbox_MasterFloater.size[1], (materialToolbox_MasterFloater.size[2] + 260)]
			if state == false then
				materialToolbox_MasterFloater.size =[materialToolbox_MasterFloater.size[1], (materialToolbox_MasterFloater.size[2] - 260)]
		)
	) -- end of matID_rollout
	
	-----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
	
	rollout mat_rollout "Material" category:3
	(
		-- LAYOUT UI
		
		--label material_lbl "Mat:" across:2 height:20 align:#left
		materialbutton material_matBtn "None" border:true width:140 height:30 --offset:[-20,-5]
		button getMaterial_btn "Get Mat of Selection" width:180
		
		button applyMatToObjects_btn "Apply Material" height:35 width:180 offset:[0,10]
		checkbox excludeHiddenObjects_mat_cbox "Exclude Hidden Objects?" checked:true align:#center
		
		button selectObjects_mat_btn "Select Objects with Material" height:30 width:180 offset:[0,10]
		checkbox appendToSelection_mat_cbx "Append to Current Selection?" checked:false align:#center
		
		-- EVENT HANDLING
		on material_matBtn rightclick do
		(
			material_matBtn.material = undefined
		)
		
		on getMaterial_btn pressed do
		(
			try material_matBtn.material = selection[1].material
			catch()
		)
		
		on applyMatToObjects_btn pressed do
		(
			if excludeHiddenObjects_mat_cbox.checked then objArray = (for obj in selection where not obj.isHidden collect obj)
			else objArray = selection
			
			with undo on
			(
				for obj in objArray do
				(
					try
					(
						if obj.isHidden and excludeHiddenObjects_mat_cbox.checked then ()
						else obj.material = material_matBtn.material
					)
					catch()
				)
			)
		)
		
		on selectObjects_mat_btn pressed do
		(
			if excludeHiddenObjects_mat_cbox.checked then objArray = (for obj in objects where not obj.isHidden collect obj)
			else objArray = objects
			
			if appendToSelection_mat_cbx.checked then selectMore (for obj in objArray where obj.material == material_matBtn.material collect obj)
			else select (for obj in objArray where obj.material == material_matBtn.material collect obj)
		)
		
		
		on mat_rollout rolledUp state do
		(
			if state == true then
				materialToolbox_MasterFloater.size =[materialToolbox_MasterFloater.size[1], (materialToolbox_MasterFloater.size[2] + 200)]
			if state == false then
				materialToolbox_MasterFloater.size =[materialToolbox_MasterFloater.size[1], (materialToolbox_MasterFloater.size[2] - 200)]
		)
	) --end of mat_rollout
	
	-----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
	
	rollout wireframeColor_rollout "Wireframe Color" category:4
	(
		-- LAYOUT UI
		colorpicker wireFrameColor_cpicker "" align:#center fieldwidth:80 color:[0,0,0]
		button getColorFromSelected_btn "Get Color of Selection" width:180
		
		button applyColorToObjects_btn "Apply Color" height:35 width:180 offset:[0,10]
		checkbox excludeHiddenObjects_color_cbox "Exclude Hidden Objects?" checked:true align:#center
		
		button selectObjects_color_btn "Select Objects with Color" height:30 width:180 offset:[0,10]
		checkbox appendToSelection_color_cbx "Append to Current Selection?" checked:false align:#center
		
		label space_lbl "" height:20 visible:false
		
		group "Rainbow Me"
		(
			colorpicker startColor_cpicker "Start Color: " align:#center fieldwidth:100 color:[200,0,0]
			radiobuttons rainbowMethod_radiobtn "Method:" labels:#("Material","Mat ID","Object") offset:[0,10]
			radiobuttons selectedOrScene_radioBtn "" labels:#("Selection","Scene") visible:false default:1
			button applyRainbow_btn "Apply Rainbow"  height:35 width:180 offset:[0,-15]

		)
		
		-- EVENT HANDLING
		
		on getColorFromSelected_btn pressed do
		(
			obj = selection[1]
			try wireFrameColor_cpicker.color = obj.wireColor
			catch()
		)
		
		on applyColorToObjects_btn pressed do
		(
			if excludeHiddenObjects_color_cbox.checked then objArray = (for obj in selection where not obj.isHidden collect obj)
			else objArray = selection
			
			for obj in objArray do
			(
				try obj.wireColor = wireFrameColor_cpicker.color
				catch()
			)
		)
		
		on selectObjects_color_btn pressed do
		(
			if excludeHiddenObjects_color_cbox.checked then objArray = (for obj in objects where not obj.isHidden collect obj)
			else objArray = objects
			objectsToSelect = #()
			
			try objectsToSelect = (for obj in objArray where obj.wireColor == wireFrameColor_cpicker.color collect obj)
			catch()
			
			if appendToSelection_color_cbx.checked then selectMore(objectsToSelect)
			else select(objectsToSelect)
		)
		
		on applyRainbow_btn pressed do
		(
			applyWireColorRainbow rainbowMethod_radiobtn.state selectedOrScene_radioBtn.state startColor_cpicker.color
		)
		
		on wireframeColor_rollout rolledUp state do
		(
			if state == true then
				materialToolbox_MasterFloater.size =[materialToolbox_MasterFloater.size[1], (materialToolbox_MasterFloater.size[2] + 350)]
			if state == false then
				materialToolbox_MasterFloater.size =[materialToolbox_MasterFloater.size[1], (materialToolbox_MasterFloater.size[2] - 350)]
		)
		
	) --end of wireframeColor_rollout
	


--//=================================================================================================//--	
--//======================================= EXECUTE SCRIPT ==========================================//--	
	
	try
	(
		addRollout about_rollout materialToolbox_MasterFloater rolledUp: false
		addRollout matID_rollout materialToolbox_MasterFloater rolledUp: true
		addRollout mat_rollout materialToolbox_MasterFloater rolledUp: true
		addRollout wireframeColor_rollout materialToolbox_MasterFloater rolledUp: true
		--addRollout rainbowMe_rollout materialToolbox_MasterFloater rolledUp: false
	)
	catch()
	
	
	
	
-- End of Script